<p align='center'>
<a href='https://github.com/tasjs/tas' target='_blank'><img width='150' src='./res/logo_icon_150.png' /></a><br />
<img src='./res/slogan.png' />
</p>

<p align='center'>
<a href='https://travis-ci.org/tasjs/tas' target='_blank'><img src='https://img.shields.io/travis/tasjs/tas/master.svg?label=linux' alt='Linux Build Status' /></a>
<a href='https://ci.appveyor.com/project/tasjs/tas' target='_blank'><img src='https://img.shields.io/appveyor/ci/tasjs/tas/master.svg?label=windows' alt='Windows Build Status' /></a>
<a href='https://codecov.io/gh/tasjs/tas' target='_blank'><img src='https://img.shields.io/codecov/c/github/tasjs/tas/master.svg' alt='Coverage Status' /></a>
<a href='https://npmjs.org/package/tas' target='_blank'><img src='https://img.shields.io/npm/v/tas.svg' alt='Version' /></a>
<a href='https://npmjs.org/package/tas' target='_blank'><img src='https://img.shields.io/npm/l/tas.svg' alt='License' /></a>
</p>

　

Make it easy to develop large, complex [Node.js](https://nodejs.org) app. With Tas, we are free to combine async tasks and sync tasks, all of them are performed in the order we arranged, which greatly simplifies the development, and the code is clear, and easy to maintain. Tas means "tasks".

Most people use the nested callback approach for handling asynchronous Node.js operations, this lead to an untameable callback hell. Now, with Tas, there is no callback hell any more, and it is extremely easy for writing a lot of asynchronous code. 

A blocking call can cause a significant reduction in performance. Now, with Tas, we can completely and easily use async api instead of blocking api in Node.js, which will make the overall performance of the app greatly improved.

　

## Install

```bash
$ npm install tas --save
```

　

## Test

Clone the Tas repo first:

```bash
$ git clone https://github.com/tasjs/tas.git
$ cd tas
$ npm install
```

Run the tests:
```bash
$ npm test
```

　

## Demo

![](./examples/demo/__res/demo.png)

The results are as follows:

![](./examples/demo/__res/demo-result.gif)

Try it:

```bash
$ cd /path/to/tas
$ node examples/demo/index.js
```

　

## Quick Example

```js
// Use .begin() at the beginning of main js file.
var tas = require('tas').begin();
var a;

var tasks = function(){
    
    // Put sync code before tas().
    // Or put them in tas() like doSomething() at below.
    a = 0;
    
    // Use tas() to join the task(s) into Tas queue.
    // Tas will perform it in the order we arranged 
    // instead of running it immediately.
    tas({
        doSomething: function(){
          
            a ++; // 1

            // Go to the next task
            tas.next();
        },

        waitAndContinue: function(){
            a ++; // 2

            // Simulate an async task
            setTimeout(function(){
                a ++; // 3

                // Go to the next task when this async task is done.
                tas.next(2);
            }, 1000);
        }
    });

    tas(function(a0){
        a += a0 ; // 5
        console.log(a); // 5

        // Use tas.end() at the ending of main js file.
        tas.end();
    });
};

tas(tasks);
```

　

## Full Examples

See the **examples/usage** folder. Try to run index.js and index.test.js (if exists) in each sub folder.

* [Beginning](./examples/usage/1.task-stream/beginning)
* [The order of tasks](./examples/usage/1.task-stream/the-order-of-tasks)
* [Mixing sync tasks and async tasks](./examples/usage/1.task-stream/mixing-sync-async-tasks)
* [Passing data](./examples/usage/1.task-stream/passing-data)
* [Fixing callback hell / callback pyramid with Tas](./examples/usage/1.task-stream/fixing-callback-hell-pyramid)
* --
* [Abort the current task stream](./examples/usage/2.breaking-flow/tas.abort)
* [Break the current tasks](./examples/usage/2.breaking-flow/tas.break)
* --
* [Simple log tree](./examples/usage/3.log-tree/simple)
* [Complex log tree](./examples/usage/3.log-tree/complex)


　

## API

See the **examples/api** folder. Try to run each file in each sub folder.

　

### Task Stream

| Usage       | Functions                                | Example                                  |
| ----------- | ---------------------------------------- | ---------------------------------------- |
| tas.begin() | Use it at the beginning of main js file. | [Example](./examples/api/1.task-stream/task-stream.js) |
| tas()       | Add an async task(s) or a sync task(s) to [task stream](./doc/execution-order/concurrency-order.md). | [Example](./examples/api/1.task-stream/task-stream.js) |
| tas.next()  | Go to the next task. Use tas.pass() to [improve performance](./doc/go-to-the-next-task/index.md). | [Example](./doc/go-to-the-next-task/index.md) |
| tas.end()   | Use it at the ending of main js file.    | [Example](./examples/api/1.task-stream/task-stream.js) |

　

### Breaking Flow

| Usage       | Functions                                | Example                                  |
| ----------- | ---------------------------------------- | ---------------------------------------- |
| tas.abort() | Abort the current [task stream](./doc/execution-order/concurrency-order.md). | [Example](./examples/api/2.breaking-flow/tas.abort.js) |
| tas.break() | Break the current tasks.                 | [Example](./examples/api/2.breaking-flow/tas.break.js) |

　

### Log Tree

| Usage             | Functions                                | Example                            |
| ----------------- | ---------------------------------------- | ---------------------------------- |
| tas.enableTree()  | Enable printing log tree.                | [Example](./doc/log-tree/index.md) |
| tas.disableTree() | Disable printing log tree.               | [Example](./doc/log-tree/index.md) |
| tas.log()         | Print custom log with indent of log tree. | [Example](./doc/log-tree/index.md) |
| tas.load()        | Print log tree with header and footer in submodule. | [Example](./doc/log-tree/index.md) |


　

## Performance

Tas can performs more than **1 million** async tasks per second. With Tas, we can write server app with clear structure and excellent performance in Node.js. [See details](./doc/performance/index.md)

![](./doc/performance/__res/tas-vs-promise-in-node.js-v8.png)
　

## License

[MIT](LICENSE)

Copyright (c) 2017, Owen Luke
