
## The Concurrency Order

In the section "[Execution Order](./index.md)", we know how Tas works, now let's see how Tas performs concurrent tasks. The concurrent tasks are occurs in sequence. Tas performs all tasks in each concurrency as a **task stream**, and 100 times concurrency correspond to 100 task streams. 

Consider the following code:

```js
var tas = require('tas');
var tester = function(done, count){

    tas(function(){
        process.nextTick(function(){
            tas.next();
        });
    });

    tas(function(){
        done(count);
        tas.end();
    });
};
```

If the tester() is executed 3 times, the map of the task streams will like below:

![](./__res/concurrency-order.png)

The variable count records the serial number of each task stream. When the current task stream execution is completed, done() is called and pass the value of variable count, it must matches the value passed when tester() is called, otherwise it indicates that the current task stream is failed.

On the other hand, we can think of the variable count as the context of the execution of the task stream, which means that Tas saved the environment variables for each task stream correctly, and that all tasks get the correct environment variables.

Try it:

```bash
$ cd /path/to/tas
$ node benchmark/analytics/concurrency-order/tas.js
```

The result will be like below:

![](./__res/concurrency-order-result.png)
　

### Abort Tas

What happens if we abort Tas in some of task stream? Well, the remaining tasks in the current tasks stream will be ignored, the other task streams will be performs one by one, that's what we expected. For example:

```js
a = 1;

tas({
    t1: {
        t11: function(){
            a ++; // 2
            tas.next();
        },

        t12: function (){
            [1].forEach(function(){

                // Abort Tas, then the remaining tasks will be ignored.
                tas.abort();
            });
        },

        t13: function(){ // ignored
            a ++; // 3
            tas.next();
        },
    },

    t2: function (){ // ignored
        a ++; // 4
        tas.next();
    }
});

tas(function(){ // ignored
    a ++; // 5
    console.log(a);
    tas.next();
});
```

Try it:

```bash
$ cd /path/to/tas
$ node examples/usage/2.breaking-flow/tas.abort/index.test.js
```

