## The Tasks Execution Order

### With Promise (Not Perfect)

Think about the following code:

```js
var tester = function(){
    console.log(0);

    new Promise(function (resolve, reject) {
        console.log(1);

        process.nextTick(function () {
            console.log(2);

            resolve();
        });
    }).then(function (data) {
        console.log(3);

    }).then(function (data) {
        console.log(4);
    });
};
```

When the tester() execution is completed, what is the number order in the console?

Yes, obviously, it's 0, 1, 2, 3, 4.

　

Now, what happens if the tester() is executed twice concurrently?

Unexpectedly, the order is 0, 1, 0, 1, 2, 2, 3, 3, 4, 4, **NOT** 0, 1, 0, 2, 3, 4, 1, 2, 3, 4.

What ever native Promise or promise library, they have the same problem. 

　

Try it:

```bash
$ cd /path/to/tas
$ node benchmark/analytics/execution-order/native-promise.js
$ node benchmark/analytics/execution-order/bluebird.js
$ node benchmark/analytics/execution-order/co.js
```

　

### With Tas (Perfect)

In Tas, there is no such problem. Let's rewrite the above code with Tas:

```js
var tester = function(){
    console.log(0);

    tas(function(){
        console.log(1);

        process.nextTick(function(){
            console.log(2);

            tas.next();
        });
    });

    tas(function(err, data){
        console.log(3);
        tas.next();
    });

    tas(function(){
        console.log(4);
        tas.end();
    });
};
```

Now, what happens if the tester() is executed 3 times concurrently?

The order will be 0, **1**, 0, 0, **2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4**, so perfect!

![](./__res/execution-order-result.png)
　

Try it:

```bash
$ cd /path/to/tas
$ node benchmark/analytics/execution-order/tas.js
```

　

Wait, what are these **0, 1, 0, 0** going on? Well, let's explain it.

In fact, the concurrent tasks are occurs in sequence, but the duration is very short. We say that the tester() is executed 5 times concurrently, actually the tester() is executed 5 times consecutively and sequentially.

First time running, tester() is executed, **0** is printed. Then, tas() is executed immediately, **1** is printed. 'Cause there is no tas.next() to go to the next task immediately, Tas will be hangged up, so there is no 2, 3, 4 are printed.

Second time running, tester() is executed, **0** is printed. And, tas() and other tasks will be added to queue directly, so there is no 1, 2, 3, 4 are printed, too. And the 3rd time running, **0** is printed.

Now, all tester() executions are completed, the current [event loop](https://nodejs.org/en/docs/guides/event-loop-timers-and-nexttick/) is done, JavaScript engine will perform the next event, the first async task in tas(), **2** is printed. Then, tas.next() returns the results, the tasks in queue is executed one by one, so **3, 4** are printed. 

Then, the other tasks which added to queue in 2nd, 3rd time running is executed one by one. so, we can see the **1, 2, 3, 4** are printed consecutively 2 times. Cool!

See [Concurrency Order](./concurrency-order.md) to learn more.

