## Go To The Next Task

　

### Use tas.next()

In general, we can simply use tas.next() to go to the next task, whatever it is in sync task or async task, like below:

```js
// A sync task
tas(function(){
    a ++;
  
    // In sync task, we should use tas.next() to go to the next task.
    tas.next();
});

tas(function(){
  
    // An async task
    setTimeout(function(){
        a ++;
      
        // Go to the next task when this async task is done.
        tas.next();
    }, 0)
  
    // Note that here is no tas.next(). 
});
```

Note the last comment "here is no tas.next()". Due to that reason, Tas will be hangged up,  until the async task is done and tas.next is performed. So, **DO NOT** add tas.next() repeatedly in there.

For the same reason, when we call other modules in the task, the current task does not need to add tas.next() repeatedly, but the called module needs to add tas.next() at the end. See below.

　

**a.js (dependency)**

```js
var tas = require('tas');

var tasks = function(){
    tas(function(){
        a ++; // 3
        setTimeout(function(){
            a ++; // 4
          
            // Go to the next task, 
            // even if it is in another module.
            tas.next();
        }, 0);
    });
};

var exports = {
    do: function(){
        tas(tasks);
    }
};

module.exports = exports;
```

　

**index.js (denpendent)**

```js
var tas = require('tas').begin();
var ma = require('./a');

var tasks = function(){

    // Define the a as global varialble for testing.
    global.a = 0;

    tas(function(){
        a ++; // 1

        setTimeout(function(){
            a ++; // 2
            tas.next();
        }, 0);
    });

    tas(function(){

        // Note that there is a tas.next() at the ending of ma.do(),
        // so here we should not put a tas.next() once again.
        ma.do();
    });

    tas(function(){
        a ++; // 5
        console.log(a); // 5
        tas.end();
    })
};

tas(tasks);
```

　

Try it:
```bash
$ cd /path/to/tas
$ node examples/usage/3.log-tree/complex/index.js
```

　

### Use tas.pass() or tas.pass2() in async task

If you extremely care about the performance, please use tas.pass() in async tasks. Tas.pass() is faster than tas.next(), because tas.next() will be executed at the next tick, and tas.pass() will be executed immediately. like below:

```js
tas(function(){
    a ++;
  
    // Must use tas.next() in sync task.
    tas.next();
});

tas(function(){
  
    // An async task
    setTimeout(function(){
        a ++;
      
        // Use tas.pass() instead of tas.next() in async task.
        tas.pass();
    }, 0)
});

tas(function(){
    console.log(a);
    tas.next();
});
```

　

If you only pass 2 parameters like (err, data), please use tas.pass2(), it is faster than tas.pass(), because it handles only 2 parameters. like below:

```js
tas(function(){
    a ++;
  
    // Must use tas.next() in sync task.
    tas.next();
});

tas(function(){
  
    // An async task
    setTimeout(function(err, data){
        a ++;
      
        // Use tas.pass2() instead of tas.next() in async task.
        tas.pass2(err, data);
    }, 0);
});

tas(function(err, data){
    console.log(data);
    tas.next();
});
```

　

### The performance comparison of tas.next() and tas.pass()

We can see that tas.pass2() is faster than tas.pass(), and tas.pass() is faster than tas.next(). But, if your server app have many async tasks, the performance difference between them is very small.

![](../performance/__res/tas.next-vs-tas.pass.png)

Try it:

```bash
$ cd /path/to/tas
$ node benchmark/tas.js
```

