/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

// Use .begin() at the beginning of main js file (in this case, it is this file).
// It is not necessary for submodules.
var tas = require('../tas').begin();
var a;

// In general, we can simply use tas.next() to go to the next task,
// whatever it is in sync task or async task.

// If you want to pursue the ultimate performance, you should use
// tas.pass() or tas.pass2() instead of tas.next() in async tasks.

// 1. tas.pass() is faster than tas.next().
// Because tas.next() will be executed at the next tick,
// and tas.pass() will be executed immediately.

// 2. tas.pass2() is faster than tas.pass().
// Because tas.pass2() handles only 2 parameters.
// The tas.pass2() applies to pass the parameters like (err, data).

var tasks = function(){

	a = 1;

	tas({
		t1: function(){
			a ++; // 2
			tas.next();
		},

		t2: function(){
			a ++; // 3

			setTimeout(function(){
				a ++; // 4

				// Pass a parameter
				tas.pass(1);
			}, 0);
		},

		t3: function(a0){
			a += a0; // 5

			setTimeout(function(){

				// Pass multiple parameters
				tas.pass(2, 3, 4);
			}, 0);
		}
	});

	tas({
		t1: function (a0, a1, a2){
			a += a0; // 7
			a += a1; // 10
			a += a2; // 14

			setTimeout(function(){

				// Pass nothing, just go to the next task.
				tas.pass();
			}, 0);
		},

		t2: function (arr){
			a ++; // 15
			tas.next();
		}
	});

	tas(function(){
		console.log(a); // 15

		// Use tas.end() at the ending of main js file.
		tas.end();
	});
};

tas(tasks);
