/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../tas');
var a;

var tasks = function(){

	// In Tas:
	// 	1. A function is called a task;
	// 	2. An object is called a tasks, 'cause its a series of task.

	// The tas() just joins the task(s) to Tas queue.
	// Tas will perform it in the order we arranged
	// instead of running it immediately.

	// Put sync code before tas().
	// Or put them in tas() like doSomething() at below.
	a = 0;

	// Add a task to Tas queue.
	tas(function doSomething(){
		a ++; // 1

		// Go to the next task.
		tas.next();
	});

	// Add a tasks to Tas queue.
	tas({
		t1: function(){
			a ++; // 2

			// Simulate an async task.
			setTimeout(function(){
				a ++; // 3

				// Go to the next task when this async task is done.
				tas.next();
			}, 0);
		},

		t2: function(){
			a ++; // 4

			setTimeout(function(){
				a ++; // 5
				tas.next();
			}, 0);
		},

		t3: {
			t31: function(){
				a ++; // 6

				setTimeout(function(){
					a ++; // 7
					tas.next();
				}, 0);
			},

			t32: function(){
				a ++; // 8

				setTimeout(function(){
					a ++; // 9
					tas.next();
				}, 0);
			},
		}
	});

	tas(function(){
		a ++; // 10
		tas.next();
	});

	tas(function(){
		console.log(a); // 10
		tas.next();
	});
};

var exports = {
	do: function(){
		tas(tasks);
	}
};

module.exports = exports;
