/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../../tas');
var fs = require('fs');

var tasks = function(){

	//--------------------------------------------
	// An example of callback hell with async file operation, thanks to Scott Robinson:
	// http://stackabuse.com/avoiding-callback-hell-in-node-js/
	//--------------------------------------------

	//var fs = require("fs")
	//var myFile = "/tmp/test"
	//var txt = "An example of callback hell in NodeJS."
	//
	//fs.writeFile(myFile, txt, function(err) {
	//	if(err) {
	//		return console.log(err);
	//	}
	//
	//	fs.readFile(myFile, "utf8", function(err, txt) {
	//		if(err) {
	//			return console.log(err);
	//		}
	//
	//		txt = txt + "\\nAppended something!";
	//		fs.writeFile(myFile, txt, function(err) {
	//			if(err) {
	//				return console.log(err);
	//			}
	//
	//			console.log("Appended text!");
	//		});
	//	});
	//});
	//--------------------------------------------

	// Let's use Tas to fix it, again.

	var file = __dirname + '/__file.tmp';
	var text = 'Using Tas to fix callback hell.';

	tas({
		init: function(){
			console.log(++ a); // 4

			fs.writeFile(file, text, 'utf8', function(err) {
				if (err) return tas.break(err);
				tas.next();
			});
		},

		readFile: function() {
			console.log(++ a); // 5

			fs.readFile(file, 'utf8', function(err, txt) {
				if (err) return tas.break(err);
				text = txt;
				tas.next();
			});
		},

		setText: function() {
			console.log(++ a); // 6

			text += '\nAppended something.';
			tas.next();
		},

		writeFile: function() {
			console.log(++ a); // 7

			fs.writeFile(file, text, 'utf8', function(err) {
				if (err) return tas.break(err);
				tas.next();
			});
		}
	});

	tas(function(){
		console.log(++ a); // 8
		tas.next();
	});
};

var exports = {
	do: function(){
		tas(tasks);
	}
};

module.exports = exports;
