/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../tas');

var env = {};
var a;

var tasks = function(){
	if (env.count === 2) return tas.abort('---- abort at 2nd');

	a = 0;

	tas({
		t1: {
			t11: function(){
				a ++; // 1
				tas.next();
			},

			t12: function (){
				[1].forEach(function(){

					// The remaining task(s) in the current tasks will be ignored.
					// Note that it is only valid for the current object (in this case, it is "t1").
					// Just like the statement "break" can only quit the current loop.
					tas.break();
				});
			},

			t13: function(){ // ignored
				a ++;
				tas.next();
			},
		},

		t2: function (){
			a ++; // 2
			tas.next();
		}
	});

	tas(function(){
		a ++; // 3
		console.log(a);
		tas.next();
	});
};

var exports = {
	do: function(count){

		// Save the environment variables to this module.
		// Note that we should place this operation in a function and
		// join it into Tas queue, otherwise the environment variables
		// will be changed immediately instead of waiting for its turn.
		tas(function(){
			env.count = count;
			tas.next();
		});

		// Join the tasks to Tas queue.
		// Tas will perform it in the order we arranged
		// instead of running it immediately.
		tas(tasks);

		// For require(..).do()
		return this;
	},

	get: function(){
		return a; // 3
	}
};

module.exports = exports;
