/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

'use strict';

const
	data = require('./data'),
	units = require('./units');

const
	flag = data.flag,
	dataUnits = data.units,
	dataTasks = data.tasks,
	ext = data.extensions;

let	tree;

const
	runner = {
	init(Tas){
		tree = Tas.tree;
	},

	exec(queue){

		/* istanbul ignore next */
		if (flag.isAwait) return;

		ext.isTreeEnabled ?
			this.execForTree(queue) :
			this.runFunc(queue.shift());
	},

	execForTree: function(queue){
		while(!flag.isAwait) {
			this.runFunc(queue.shift());
		}
	},

	runFunc(func){
		ext.isTreeEnabled && tree.print(func);

		data.maxLayer = data.layer;
		data.layer ++;

		data.pass.isHasArgs ?
			func.apply(null, data.pass) :
			func();

		flag.isAwait = true;
		data.layer > 0 && data.layer --;
	},

	next(){
		let lay = data.maxLayer;
		let task;

		flag.isAwait = false;
		while(lay >= 0) {

			data.layer = lay;

			// Handle the remain functions
			!flag.isAwait && dataUnits[lay] && (task = dataUnits[lay].shift()) && (
				this.runFunc(task)
			);

			// Handle the tasks in sequence
			!flag.isAwait && dataTasks[lay] && (task = dataTasks[lay].shift()) && (
				typeof task === 'function' ?
					this.runFunc(task) :
					units.do(task)
			);

			lay --;
		}

		// When all tasks done, then restore flag isAwait to false
		(!dataTasks[0] || !dataTasks[0].length) && (flag.isAwait = false);
	}
};

module.exports.__proto__ = runner;
