/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

'use strict';

const
	data = require('./data'),
	runner = require('./runner'),
	units = require('./units');

const
	flag = data.flag,
	dataTasks = data.tasks,
	dataUnits = data.units;

const
	tasks = {
	add(obj){

		// A function
		if (typeof obj === 'function' && !data.layer && !flag.isAwait) {
			runner.runFunc(obj);
		}
		else {
			// An object (a series of functions)
			let layer = data.layer;
			obj.layer = layer;

			typeof dataTasks[layer] === 'undefined' && (dataTasks[layer] = []);
			dataTasks[layer].push(obj);

			// Try to run the next task
			!flag.isAwait && units.do(dataTasks[data.layer].shift());
		}
	},

	clearTheRemainingTasks(){
		let maxLayer = data.maxLayer;
		let lay;

		// clear the remaining functions in this tasks
		for (lay = maxLayer; lay >= 0; lay --) {
			dataUnits[lay] && (dataUnits[lay].length = 0);
		}

		// clear the child tasks in this tasks
		/* istanbul ignore next */
		for (lay = maxLayer; lay > 0; lay --) {
			dataTasks[lay] && (dataTasks[lay].length = 0);
		}

		// clear the remaining tasks in this task stream
		let tasks = dataTasks[0];
		while (tasks && tasks.length) {
			if (tasks[0].isStreamBegin) break;
			tasks.shift();
		}
	}
};

module.exports.__proto__ = tasks;
