/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

'use strict';

const
	data = require('./data'),
	runner = require('./runner');

const
	dataUnits = data.units,
	ext = data.extensions,
	currentTask = {};

let tree;

const
	units = {
	init(Tas){
		tree = Tas.tree;
	},

	do(task){
		this.add(task);
		this.run(task);
	},

	add(task){
		let layer = task.layer || 0;

		currentTask.layer = layer;
		currentTask.root = task;

		typeof dataUnits[layer] === 'undefined' && (dataUnits[layer] = []);

		let arr = dataUnits[layer];
		ext.isTreeEnabled ?
			this.addForTree(layer, task, arr) :

			typeof task === 'function' ?
				arr.push(task) :
				this.getAllFunctions(layer, task, task, arr, 0);
	},

	addForTree(layer, task, arr){
		let t = task;
		let o = {};

		if (typeof task === 'function') {
			o[task.name] = task;
			t = o;
		}

		tree.nested.createFnForBegin(arr);
		this.getAllFunctions(layer, t, t, arr, 0);
		tree.nested.createFnForEnd(arr);
	},

	run(task){
		let queue = dataUnits[task.layer];
		queue && queue.length && runner.exec(queue);
	},

	getAllFunctions(layer, root, obj, arr, deep) {
		let keys = Object.keys(obj),
			isTreeEnabled = ext.isTreeEnabled;

		for (let i = 0, len = keys.length; i < len; i ++) {
			let key = keys[i],
				func = obj[key],
				type = typeof func;

			if (type === 'object') {
				if (isTreeEnabled) {
					tree.nested.createFnForAdd(key, arr, deep);
					this.getAllFunctions(layer, root, func, arr, deep + 1);
					tree.nested.createFnForSub(arr, deep);
				}
				else {
					this.getAllFunctions(layer, root, func, arr, deep + 1);
				}
			}
			else
			if (type === 'function') {
				func.root = root;
				func.layer = layer;

				arr.push(func);

				if (isTreeEnabled) {
					func.treeBranch = key;
					func.treeIndent = tree.nested.getCount() + deep;
				}
			}
		}
	},

	clearTheRemainingFunctions(){
		let layer = currentTask.layer,
			root = currentTask.root,
			functions = dataUnits[layer];

		for (let i = 0, len = functions.length; i < len; i ++) {
			functions[i] && functions[i].root === root && functions.shift();
		}
	}
};

module.exports.__proto__ = units;
