/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

'use strict';

const
	data = require('../../core/data'),
	nested = require('./nested'),
	logArray = require('./logArray'),
	indent = require('./indent'),
	submodule = require('./submodule'),
	print = require('./print'),
	util = require('./util');

const
	ext = data.extensions;

let Tas;

const
	tree = {
	init(Tas_){
		Tas = Tas_;
		submodule.saveTas(Tas_);

		this.nested = nested;

		Tas.tree = this;
		Tas.enableTree = this.enable;
		Tas.disableTree = this.disable;

		Tas.log = this.log;
		Tas.logArray = logArray;
	},

	enable(){
		ext.isTreeEnabled = true;
		return Tas;
	},

	disable(){
		ext.isTreeEnabled = false;
		return Tas;
	},

	wrapModule(filename, obj){
		submodule.wrapWithHeaderAndFooter(filename, obj);
	},

	print(func){
		print.do(func);
	},

	log(){
		let args = Array.prototype.slice.call(arguments);
		let indentStr = indent.getStr(true);

		args.unshift(indentStr);
		util.log.apply(null, args);
	},

	reset(){
		indent.reset();
		nested.reset();
	}
};

module.exports = tree;
