/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

'use strict';

const
	moduleIndent = require('./moduleIndent'),
	tasks = require('../../../core/tasks'),
	runner = require('../../../core/runner'),
	indent = require('../indent'),
	util = require('../util');

const
	cutLine = util.repeat('─', 4),
	indentStrings = {};

let	levelCount = 0,
	appPath = '',
	Tas;

const
	submodule = {
	wrapWithHeaderAndFooter(filename, obj){
		levelCount ++;

		this.addPrintingHeader(filename);
		this.addTasks(obj);
		this.addPrintingFooter();
	},

	addPrintingHeader(filename){
		!appPath && tool.getAppPath(filename);

		let thisFileName = filename
			.replace(appPath, '') // Remove root path
			.replace(/\\/g, '/'); // Compatible with Windows

		let header = '┌' + cutLine + ' ' + thisFileName + ' ' + cutLine;
		let printHeader = function(){
			process.nextTick(function(){
				runner.next();
			});
		};

		printHeader.__label = header;
		printHeader.__isHeader = true;
		printHeader.__levelCount = levelCount;

		tasks.add(printHeader);
	},

	addTasks(obj){
		obj.__label = 'fn';
		tasks.add(obj);
	},

	addPrintingFooter(){
		let footer = '└' + cutLine;
		let printFooter = function(){
			process.nextTick(function(){
				runner.next();
			});
		};

		printFooter.__label = footer;
		printFooter.__isFooter = true;
		printFooter.__levelCount = levelCount;

		tasks.add(printFooter);
	},

	getIndentStrWithModuleIndent(func){
		var indentStr = indent.getStr();

		// Indent for header and footer
		func.__isHeader ? indentStrings[func.__levelCount] = indentStr : 0;
		func.__isFooter ? indentStr = indentStrings[func.__levelCount] : 0;

		func.__isHeader ? moduleIndent.push(indentStr) : 0;
		func.__isFooter ? moduleIndent.pop() : 0;

		return indentStr;
	},

	insertModuleIndent(str){
		return moduleIndent.insertModuleIndentStr(str);
	},

	saveTas(Tas_){
		Tas = Tas_;
	}
};

const
	tool = {
	getAppPath(filename){
		let char = filename.indexOf('/') >= 0 ? '/' : '\\'; // Compatible with Windows
		let index = filename.lastIndexOf(char, filename.lastIndexOf(char) - 1);
		appPath = filename.substr(0, index);
	}
};

module.exports = submodule;
