/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

'use strict';

const
	data = require('./core/data'),
	runner = require('./core/runner'),
	tasks = require('./core/tasks'),
	pass = require('./core/pass'),
	abort = require('./core/abort'),
	break_ = require('./core/break'),
	begin = require('./core/begin'),
	prepare = require('./prepare');

const
	slice = Array.prototype.slice,
	nextTick = process.nextTick,
	ext = data.extensions,
	beginTry = begin.try,
	beginSet = begin.set,
	addTasks = tasks.add,
	pass2 = pass.save2,
	pass1 = pass.save1,
	pass0 = pass.save0,
	passArray = pass.saveArray,
	breakDo = break_.do,
	abortDo = abort.do;

let Tas;

Tas = function(obj){
	beginTry(obj);
	addTasks(obj);
};

Tas.next = function(a0, a1){
	let args = arguments;
	nextTick(() => {
		let len = args.length;

		len === 2 ? pass2(a0, a1) :
			len === 1 ? pass1(a0) :
				len === 0 ? pass0() :
					passArray(slice.call(args));

		runner.next();
	});
};

Tas.pass = function(a0, a1){
	let len = arguments.length;

	len === 2 ? pass2(a0, a1) :
		len === 1 ? pass1(a0) :
			len === 0 ? pass0() :
				passArray(slice.call(arguments));

	runner.next();
};

Tas.pass2 = function(err, data){
	pass2(err, data);
	runner.next();
};

Tas.break = function(){
	let args = arguments;
	nextTick(() => {
		breakDo(slice.call(args));
	});
};

Tas.abort = function(){
	let args = arguments;
	nextTick(() => {
		abortDo(slice.call(args));
	});
};

Tas.begin = function(){
	beginSet();
	return Tas;
};

Tas.end = function(){
	runner.next();
};

Tas.load = function(obj){
	ext.isTreeEnabled ?
		Tas.tree.wrapModule(require('parent-module')(), obj) :
		addTasks(obj);
};

// API of extensions - tree
Tas.tree = undefined;
Tas.enableTree = undefined;
Tas.disableTree = undefined;

// API of extensions - log
Tas.log = undefined;
Tas.logArray = undefined;

// Load extensions
prepare.do(Tas);

module.exports = Tas;
