/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tester = require('../tester');
var expect = require('chai').expect;

describe('1.task_stream: tas.pass()', function(){
	it('should return 20,19', function(done){

		var test = function(done, count){

			var tas = require('../tas').begin();
			var a = 1;

			tas({
				t1: function(){
					a ++; // 2
					tas.next();
				},

				t2: function(){
					if (count === 1) {
						a ++; // 3
					}

					setTimeout(function(){
						a ++; // 4

						// Pass a parameter
						tas.pass(1);
					}, 0);
				},

				t3: function(a0){
					a += a0; // 5

					setTimeout(function(){

						// Pass multiple parameters
						tas.pass(2, 3, 4);
					}, 0);
				}
			});

			tas({
				t1: function (a0, a1, a2){
					a += a0; // 7
					a += a1; // 10
					a += a2; // 14

					setTimeout(function(){

						// Pass an array parameter.
						var arr = [1, 2, 3];
						tas.pass(arr);
					}, 0);
				},

				t2: function (arr){
					arr.forEach(function(e){
						a += e; // 20
					});

					setTimeout(function(){

						// Pass nothing, just go to the next task.
						tas.pass();
					}, 0);
				}
			});

			tas(function (){
				done(count, a);
				tas.end();
			});
		};

		var check = function(results){
			expect(results.toString() === '20,19').to.be.equal(true);
			done();
		};

		// Run the test twice
		tester.do(test, check, 2);
	});
});
