/**
 * Test of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tester = require('../tester');
var expect = require('chai').expect;

var r1 = [];

describe('3.log-tree: complex', function(){

	it('should create tree logs', function(done){

		var test = function(done, count) {

			var tas = require('../tas').begin();
			var app = require('./complex/app').do(count);

			tas(function (){
				done(count, app.get());
				tas.end();
			});
		};

		var check = function(results){
			expect(results.toString() === ',' + r1.join('\n')).to.be.equal(true);
			done();
		};

		// Run the test twice
		tester.do(test, check, 2);
	});
});

r1 = [
	' 1| <anonymous>',
	' 1| ┌──── /modules/index.js ────',
	' 1| │ m1',
	' 2| │     ┌──── /modules/m1/index.js ────',
	' 2| │     │ t1',
	' 3| │     │     ┌──── /modules/m1/ma/index.js ────',
	' 3| │     │     │ t2',
	' 4| │     │     │     t3',
	' 5| │     │     │         wait...',
	' 5| │     │     │         t4',
	' 5| │     │     │         continue...',
	' 4| │     │     │     t5',
	' 4| │     │     │     t6',
	' 5| │     │     │         t7',
	' 5| │     │     │         t8',
	' 4| │     │     │     t9',
	' 4| │     │     │     t10',
	' 3| │     │     │ t11',
	' 3| │     │     │ t12',
	' 3| │     │     └────',
	' 3| │     │     ┌──── /modules/m1/mb/index.js ────',
	' 3| │     │     │ t13',
	' 3| │     │     │ t14',
	' 4| │     │     │     ┌──── /modules/m1/mb/a.js ────',
	' 4| │     │     │     │ passData',
	' 4| │     │     │     │ t15 	// 1, 2, 3',
	' 4| │     │     │     │ t16 	// [4,5], {"hi":"hi"}',
	' 4| │     │     │     └────',
	' 3| │     │     │ t17',
	' 3| │     │     │ t18',
	' 3| │     │     └────',
	' 2| │     │ t19',
	' 2| │     │ t20',
	' 2| │     └────',
	' 1| │ m2',
	' 2| │     ┌──── /modules/m2/index.js ────',
	' 2| │     │ t21',
	' 3| │     │     ┌──── /modules/m2/ma/index.js ────',
	' 3| │     │     │ t22',
	' 4| │     │     │     t23',
	' 5| │     │     │         t24',
	' 5| │     │     │         t25',
	' 4| │     │     │     t26',
	' 3| │     │     └────',
	' 4| │     │         ┌──── /modules/m2/mb/index.js ────',
	' 3| │     │         │ t27',
	' 4| │     │         │     ┌──── /modules/m2/mb/a.js ────',
	' 4| │     │         │     │ t28',
	' 4| │     │         │     │ t29',
	' 4| │     │         │     └────',
	' 4| │     │         └────',
	' 3| │     │     t30',
	' 3| │     │     t31',
	' 2| │     └────',
	' 1| │ m3',
	' 2| │     ┌──── /modules/m3/index.js ────',
	' 2| │     │ t32',
	' 3| │     │     ┌──── /modules/m3/ma.js ────',
	' 4| │     │     │     wait...',
	' 4| │     │     │     t33',
	' 4| │     │     │     continue...',
	' 3| │     │     └────',
	' 2| │     │ t34',
	' 2| │     │ t35',
	' 2| │     └────',
	' 1| │ m4',
	' 2| │     ┌──── /modules/m4/index.js ────',
	' 2| │     │ t36',
	' 3| │     │     ┌──── /modules/m4/ma/index.js ────',
	' 3| │     │     │ t37',
	' 4| │     │     │     ┌──── /modules/m4/ma/ma-1/index.js ────',
	' 4| │     │     │     │ t38',
	' 5| │     │     │     │     ┌──── /modules/m4/ma/ma-1/a.js ────',
	' 5| │     │     │     │     │ <anonymous>',
	' 6| │     │     │     │     │     wait...',
	' 6| │     │     │     │     │     t39',
	' 6| │     │     │     │     │     continue...',
	' 5| │     │     │     │     └────',
	' 5| │     │     │     │     t40',
	' 4| │     │     │     └────',
	' 5| │     │     │         ┌──── /modules/m4/ma/ma-2/index.js ────',
	' 4| │     │     │         │ t41',
	' 5| │     │     │         │     wait...',
	' 5| │     │     │         │     t42',
	' 5| │     │     │         │     continue...',
	' 4| │     │     │         │ t43',
	' 5| │     │     │         └────',
	' 4| │     │     │     ┌──── /modules/m4/ma/ma-3/index.js ────',
	' 4| │     │     │     │ t44',
	' 4| │     │     │     │ t45',
	' 5| │     │     │     │     wait...',
	' 5| │     │     │     │     t46',
	' 5| │     │     │     │     continue...',
	' 4| │     │     │     │ t47',
	' 4| │     │     │     └────',
	' 3| │     │     │ t48',
	' 3| │     │     └────',
	' 3| │     │     t49',
	' 2| │     │ t50',
	' 2| │     │ t51',
	' 2| │     └────',
	' 1| │ m5',
	' 2| │     ┌──── /modules/m5/index.js ────',
	' 2| │     │ ma',
	' 3| │     │     ┌──── /modules/m5/ma/index.js ────',
	' 3| │     │     │ t0',
	' 4| │     │     │     wait...',
	' 4| │     │     │     continue...',
	' 3| │     │     └────',
	' 2| │     │ mb',
	' 3| │     │     ┌──── /modules/m5/mb/index.js ────',
	' 3| │     │     │ t1',
	' 4| │     │     │     ...',
	' 4| │     │     │     ┌──── /modules/m5/mb/mb-1/index.js ────',
	' 4| │     │     │     │ t2',
	' 5| │     │     │     │     ...',
	' 5| │     │     │     │     t21',
	' 6| │     │     │     │         ...',
	' 5| │     │     │     │     ┌──── /modules/m5/mb/mb-1/mb-1-a/index.js ────',
	' 5| │     │     │     │     │ t22',
	' 6| │     │     │     │     │     wait...',
	' 6| │     │     │     │     │     continue...',
	' 5| │     │     │     │     └────',
	' 5| │     │     │     │     t23',
	' 6| │     │     │     │         ...',
	' 4| │     │     │     │ t3',
	' 5| │     │     │     │     t31',
	' 6| │     │     │     │         ...',
	' 5| │     │     │     │     t32',
	' 6| │     │     │     │         ...',
	' 6| │     │     │     │         ┌──── /modules/m5/mb/mb-1/mb-1-b/index.js ────',
	' 6| │     │     │     │         │     t321',
	' 7| │     │     │     │         │         ...',
	' 6| │     │     │     │         │     t322',
	' 7| │     │     │     │         │         ...',
	' 6| │     │     │     │         │     t323',
	' 7| │     │     │     │         │         ...',
	' 6| │     │     │     │         └────',
	' 5| │     │     │     │     t33',
	' 6| │     │     │     │         ...',
	' 4| │     │     │     │ t4',
	' 5| │     │     │     │     ┌──── /modules/m5/mb/mb-1/mb-1-c/index.js ────',
	' 5| │     │     │     │     │ t41',
	' 6| │     │     │     │     │     ...',
	' 6| │     │     │     │     │     ┌──── /modules/m5/mb/mb-1/mb-1-c/a.js ────',
	' 6| │     │     │     │     │     │ t411',
	' 7| │     │     │     │     │     │     wait...',
	' 7| │     │     │     │     │     │     continue...',
	' 6| │     │     │     │     │     │ t412',
	' 7| │     │     │     │     │     │     ...',
	' 6| │     │     │     │     │     └────',
	' 6| │     │     │     │     │     ┌──── /modules/m5/mb/mb-1/mb-1-c/b.js ────',
	' 6| │     │     │     │     │     │ t413',
	' 7| │     │     │     │     │     │     wait...',
	' 7| │     │     │     │     │     │     continue...',
	' 6| │     │     │     │     │     └────',
	' 5| │     │     │     │     │ t42',
	' 6| │     │     │     │     │     ...',
	' 6| │     │     │     │     │     ┌──── /modules/m5/mb/mb-1/mb-1-c/c.js ────',
	' 6| │     │     │     │     │     │ t421',
	' 7| │     │     │     │     │     │     wait...',
	' 7| │     │     │     │     │     │     continue...',
	' 6| │     │     │     │     │     │ t422',
	' 7| │     │     │     │     │     │     ...',
	' 6| │     │     │     │     │     └────',
	' 5| │     │     │     │     └────',
	' 5| │     │     │     │     t43',
	' 6| │     │     │     │         ...',
	' 5| │     │     │     │     t44',
	' 6| │     │     │     │         ...',
	' 4| │     │     │     └────',
	' 4| │     │     │     t5',
	' 5| │     │     │         ...',
	' 5| │     │     │         ┌──── /modules/m5/mb/mb-2/index.js ────',
	' 5| │     │     │         │     t51',
	' 6| │     │     │         │         ...',
	' 5| │     │     │         │     t52',
	' 6| │     │     │         │         ...',
	' 5| │     │     │         │     t53',
	' 6| │     │     │         │         wait...',
	' 6| │     │     │         │         continue...',
	' 5| │     │     │         └────',
	' 4| │     │     │     t6',
	' 5| │     │     │         ...',
	' 4| │     │     │     t7',
	' 5| │     │     │         ...',
	' 3| │     │     │ t8',
	' 4| │     │     │     ...',
	' 3| │     │     └────',
	' 2| │     │ mc',
	' 3| │     │     ┌──── /modules/m5/mc/index.js ────',
	' 3| │     │     │ t9',
	' 4| │     │     │     wait...',
	' 4| │     │     │     continue...',
	' 3| │     │     └────',
	' 2| │     └────',
	' 1| └────',
	' 3|         <anonymous>'
];
