/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../tas');
var m = require('./modules');
var env = {};

// Define global variables for testing
global.a = 0;
global.timeout = 0;
global.log = tas.log;

var tasks = function(){
	if (env.count === 1) return tas.abort();

	global.a = 0;

	// Prepare
	tas(function(){

		// Enable printing log tree in this module.
		tas.enableTree();

		// Save logs into an array instead of printing it immediately.
		tas.logArray.begin();

		tas.next();
	});

	// Do something
	tas(function(){

		// Perform the method of submodule.
		m.do();

		// Note that there is a tas.next() at the ending of m.do(),
		// so here we should not put a tas.next() once again.
	});

	// The end
	tas(function(){

		// Disable printing log tree to avoid affect the other modules.
		tas.disableTree();

		tas.next();
	});
};

var exports = {
	do: function(count){

		// Save the environment variables to this module.
		// Note that we should place this operation in a function and
		// join it into Tas queue, otherwise the environment variables
		// will be changed immediately instead of waiting for its turn.
		tas(function(){
			env.count = count;
			tas.next();
		});

		// Join the tasks to Tas queue.
		// Tas will perform it in the order we arranged
		// instead of running it immediately.
		tas(tasks);

		// For require(..).do()
		return this;
	},

	get: function(){

		// Get logs from array and convert to string.
		return tas.logArray.getStr();
	}
};

module.exports = exports;