/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../tas');

var env = {};
var a;

var tasks = function(){
	if (env.count === 1) return tas.abort();

	a = 1;

	tas({
		t1: function(){
			a ++; // 2
			tas.next();
		},

		t2: function(){
			a ++; // 3

			setTimeout(function(){
				a ++; // 4

				// Pass a parameter
				tas.next(1);
			}, 0);
		},

		t3: function(a0){
			a += a0; // 5

			setTimeout(function(){

				// Pass multiple parameters
				tas.next(2, 3, 4);
			}, 0);
		}
	});

	tas({
		t1: function (a0, a1, a2){
			a += a0; // 7
			a += a1; // 10
			a += a2; // 14

			setTimeout(function(){
				var arr = [1, 2, 3];

				// Pass an array parameter.
				tas.next(arr);
			}, 0);
		},

		t2: function (arr){
			arr.forEach(function(e){
				a += e;
			});

			setTimeout(function(){

				// Pass nothing, just go to the next task.
				tas.next();
			}, 0);
		}
	});

	tas(function(){
		// console.log(a); // 20
		tas.next();
	});
};

var exports = {
	do: function(count){

		// Save the environment variables to this module.
		// Note that we should place this operation in a function and
		// join it into Tas queue, otherwise the environment variables
		// will be changed immediately instead of waiting for its turn.
		tas(function(){
			env.count = count;
			tas.next();
		});

		// Join the tasks to Tas queue.
		// Tas will perform it in the order we arranged
		// instead of running it immediately.
		tas(tasks);

		// For require(..).do()
		return this;
	},

	get: function(){
		return a; // 20
	}
};

module.exports = exports;
