/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../tas');

var env = {};
var a;
var timeout = 0;

var tasks = function(){
	if (env.count === 1) return tas.abort();

	tas(function(){

		//--------------------------------------------
		// 1. The wrong way
		//--------------------------------------------
		// The order of execution of the following code is:
		//   a = 1
		//   start()
		//   a = 2
		//   whenTimesUp()
		//   end()

		// When whenTimesUp() is called, the initial value of a is 2 (not 1).
		// So the final value of a is 3, rather than the expected 2.
		//--------------------------------------------

		a = 1;

		tas(function start(){
			setTimeout(function whenTimesUp(){
				a ++; // 3
				tas.next();
			}, 0);
		});

		a = 2;

		tas(function end(){
			// console.log(a); // It's 3, not 2.
			tas.next();
		})
	});
	
	tas(function(){

		//--------------------------------------------
		// 2. The right way
		//--------------------------------------------
		// Put the statement a = 2 into the task doIt(),
		// this ensures the correct execution order:
		//   start()
		//   whenTimesUp()
		//   doIt()
		//   end()
		//--------------------------------------------

		a = 4;

		tas(function start() {
			setTimeout(function whenTimesUp(){
				a ++; // 5
				tas.next();
			}, timeout);
		});

		tas(function doIt(){
			a = 6;
			tas.next();
		});

		tas(function end(){
			// console.log(a); // 6
			tas.next();
		});
	});


	tas(function(){

		//--------------------------------------------
		// 3. The simplified way (use anonymous function)
		//--------------------------------------------

		a = 7;

		tas(function () {
			setTimeout(function(){
				a ++; // 8
				tas.next();
			}, timeout);
		});

		tas(function(){
			a = 9;
			tas.next();
		});

		tas(function(){
			// console.log(a); // 9
			tas.next();
		});
	});
};

var exports = {
	do: function(count){

		// Save the environment variables to this module.
		// Note that we should place this operation in a function and
		// join it into Tas queue, otherwise the environment variables
		// will be changed immediately instead of waiting for its turn.
		tas(function(){
			env.count = count;
			tas.next();
		});

		// Join the tasks to Tas queue.
		// Tas will perform it in the order we arranged
		// instead of running it immediately.
		tas(tasks);

		// For require(..).do()
		return this;
	},

	get: function(){
		return a; // 9
	}
};

module.exports = exports;
